#!/usr/bin/perl


package UStream::Log;

=head1 NAME

UStream::Log - Logging functions for UStream

=head1 SYNOPSIS

use UStream::Log;

=head1 DESCRIPTION

=cut


use Exporter 'import'; # gives you Exporter import() method directly
@EXPORT = qw(getString removeSem);  # symbols to export on request

use Data::Dumper;

use IPC::SysV qw(IPC_RMID IPC_CREAT IPC_EXCL SEM_UNDO IPC_NOWAIT 
                 IPC_SET IPC_STAT SETVAL);
use IPC::Semaphore;
use POSIX qw(EEXIST);

use POSIX;
use Log::Log4perl qw(:easy);
use File::Spec::Functions;
my $INIT = 0;

my $conf = qq(
	log4perl.category			= DEBUG, Screen
	log4perl.category.UStream		= INFO, Syncer

	log4perl.appender.Screen		= Log::Log4perl::Appender::Screen
	log4perl.appender.Screen.stderr		= 0
	log4perl.appender.Screen.layout		= Log::Log4perl::Layout::SimpleLayout
	log4perl.appender.Screen.Filter		= MatchDebug

	# Filter
	log4perl.filter.MatchDebug		= Log::Log4perl::Filter::LevelMatch
	log4perl.filter.MatchDebug.LevelToMatch  = DEBUG
	log4perl.filter.MatchDebug.AcceptOnMatch = true

	# File appender (unsynchronized)
	log4perl.appender.Logfile		= Log::Log4perl::Appender::File
	log4perl.appender.Logfile.autoflush	= 1
	log4perl.appender.Logfile.filename	= _LOG_PATH_
	log4perl.appender.Logfile.mode		= append
	log4perl.appender.Logfile.layout	= Log::Log4perl::Layout::SimpleLayout

	# Synchronizing appender, using the file appender above
	log4perl.appender.Syncer		= Log::Log4perl::Appender::Synchronized
	log4perl.appender.Syncer.appender	= Logfile
	log4perl.appender.Syncer.key		= ul4p
        #log4perl.appender.Syncer.destroy	= 1

	);


=over 4

=item UStream::Log::new

Create new log file object.

=cut

sub new {
	my $class = shift;
	my $logdir = shift || "/tmp";
	my $logfile = shift || "UStream.log";

	$logdir = "/tmp" unless (-d $logdir);

	my $self = { dir => $logdir, file => $logfile };
	my $fullpath = catfile($self->{dir}, $self->{file});

	$_ = $conf;
	s/_LOG_PATH_/$fullpath/g;
	$conf = $_;

	if (! $INIT) {
	        #removeSem();
		Log::Log4perl->init(\$conf);
		$INIT = 1;
	}

	$self->{logger} = get_logger();

	bless $self, $class;

	$self->{logger}->debug("This message is guaranteed to be complete.");

	$self;
}

my $loglevels = { "fatal" => 1, "error" => 1, "warn" => 1, "info" => 1, "debug" => 1 };
sub AUTOLOAD {
	my $self = shift;
	$_ = $AUTOLOAD;
	s/.*:://;
	my $action = (/debug/) ? "debug" : shift;
	my $log_msg = shift;
	my $date = strftime("%F %T UTC",gmtime(time));
	my $user = $ENV{USER};

	my $test = "is_$_";
	if ($loglevels->{$_} && $self->{logger}->${test}()) {
		$self->{logger}->$_("date:$date|user:$user|action:$action|$log_msg");
	}

	return(0);

}

sub getString {
	$Data::Dumper::Indent = 0;
	$Data::Dumper::Terse = 1;
        sprintf("\"%s\"", Dumper(@_));
}

#::SEISMO - added this to clear semaphore
sub removeSem {
    my $key = "ul4p";
    my $ikey = unpack("i", pack("A4", $key));
    my $mode = undef;
    my $id = semget( $ikey, 1, 0 );
    if(defined $id) {
	semctl ( $id, 0, &IPC_RMID, 0 );
    }
    return;
}

=item UStream::Log::getSiteEntries

Return entries from log file that match a site.

=cut

sub getSiteEntries {
	my $self = shift;
	my $site = shift;

	my @logs;
	my $fullpath = catfile($self->{dir}, $self->{file});
	open(LOG,"<$fullpath") || return(@logs);
	while(<LOG>) {
		push(@logs,$_) if (/site..=>..$site/i);
		push(@logs,$_) if (/mount..=>..$site/i);
	}
	return(@logs);
}

__END__

=back

=head1 AUTHOR

Keith F. Stark (Stark Consulting LLC)

=head1 SEE ALSO

L<ustreamrc>, L<ustream-logger>, L<ustream.cgi>, L<UStream>,  L<UStream::CGI>, L<UStream::Log>, L<UStream::Template>

=cut

